# Soya 3D tutorial
# Copyright (C) 2001-2002 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Lesson 012: Animations : Morph -- A smiling Tux !


import soya, soya.soya3d as soya3d, soya.model as model, soya.cube as cube, soya.animation as animation

import os, os.path, sys

soya.init()

scene = soya3d.World()

data_dir = os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), "data")

model.Image   .PATH = data_dir
model.Material.PATH = data_dir
soya3d.World  .PATH = data_dir

# Loads a Tux.
# The "tux2" world is similar to "tux", but it contains 3 worlds, organized as following:
#
# the root world, with Tux, except his mouth
#  |
#  +- the left and right vertices of Tux's mouth
#  |
#  `- the upper and lower vertices of Tux's mouth
#
# After we convert this world into a morph shape, we'll still be able to move its two
# children (contrary to what happen if we use a normal shape).


tux = soya3d.World.load("tux2")


# the shapify_arg attribute can be used to control how the world is turned to a shape.
# shapify_arg is a tuple. The first element is the type of shape, and the second one is
# a dictionary with eventually key-value option pairs.
# To create a morph, we must use the "morph" type of shape. No option are available for
# morph.
#
# The magical word is:

tux.shapify_args = ("morph", {})

# Creates a morph shape

tux_morph_shape = tux.shapify()

# A morph shape cannot be used with a volume. You must put it in a morph.

tux_morph = soya3d.Morph(scene, tux_morph_shape)

tux_morph.scale(0.05, 0.05, 0.05)
tux_morph.rotate_lateral(-150.0)


light = soya3d.Light(scene)
light.set_xyz(0.0, 2.0, 2.2)

camera = soya3d.Camera(scene)
camera.y = 0.3
camera.z = 0.8

soya.set_root_widget(camera)

import time, math


# A morph is similar to a Shape, but we can access its children and move them !
# Here we move the two children in order to obtain a smiling Tux.
# The children are available as "morph.children"

tux_morph.children[1].y -= 0.8

a = 0.0
while 1:
  soya.render()
  
  a += 0.5
  delta = math.cos(a) * 0.2
  
  tux_morph.children[0].y += delta * 0.4
  tux_morph.children[1].y -= delta
  
  time.sleep(0.1)

