# Soya 3D tutorial
# Copyright (C) 2001-2002 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Lesson 014: Character animation with Cal3D : here comes the Paladin!
#
# Or how to write a Cal3D viewer in less that 20 lines...
#
# For this lesson, you must download the Cal3D example files 'cal3d_data'
# at http://cal3d.sourceforge.net


import os, os.path, sys
import soya, soya.soya3d as soya3d, soya.model as model, soya.cal3d as cal3d, soya.idler as idler, soya.widget as widget
soya.init()

scene = soya3d.World()


# You need to change this to your own "cal3d_data" directory!

model.Shape.PATH = "/home/jiba/tlchargements/cal3d_data"

if not os.path.exists(model.Shape.PATH):
  raise RuntimeError("For this lesson, you need to download and install the 'cal3d_data' package (http://cal3d.sourceforge.net), and to put the installation path in the tutorial lesson source.")

# Load a Cal3D shape.
# Cal3D shapes are saved in subdirectory of model.Shape.PATH. The subdirectory contains
# skeleton, animation, mesh and material files, and a ".cfg" file with the same name
# that the subdirectory (and and ".cfg" extension).
# See Cal3D documentation for more info, and the soya.cal3d module's docstring for
# advanced info on materials loading (e.g, how to substitute Cal3D materials by Soya
# ones).
# 
# You can also use cal3d.parse_cfg_file(filename).

paladin_shape = cal3d.Shape.get("paladin")

# You can get the list of available mesh and animation names
# as following:

print "Available mehses    :", paladin_shape.meshes
print "Available animations:", paladin_shape.animations

# Creates a Cal3D volume, using the paladin_shape.
# See the docstrings of the soya.cal3d module to learn about mesh attachment
# and detachment possibilities (see Volume.__init__, Volume.attach and
# Volume.detach). It can be used e.g. for dismembering, or changing the weapon
# of a character.

paladin = cal3d.Volume(scene, paladin_shape)

# Cal3D Paladin is really big,
# and is not oriented as Soya shapes are (e.g. Z is our Y direction)

paladin.scale(0.065, 0.065, 0.065)
paladin.rotate_vertical(-90.0)

# Starts playing the animation called "paladin_walk" in cycle.

paladin.animate_blend_cycle("paladin_walk")

# To stop playing the animation:
#
#paladin.animate_clear_cycle("paladin_walk")
#
# For non-cyclic movment, do:
#
#paladin.animate_execute_action("paladin_walk")
#
# See the cal3d.Volume.animate* docstrings for more info about optional arguments

camera = soya3d.Camera(scene)
camera.set_xyz(0.0, 2.5, 6.0)

soya.set_root_widget(widget.Group())
soya.root_widget.add(camera)
soya.root_widget.add(widget.FPSLabel())

soya3d.Light(scene).set_xyz(0.5, 5.0, 2.2)

idler.Idler(scene).idle()
